@@ -103,7 +103,6 @@ else:
         can_clip = True
     except AttributeError:  # check_call not defined, Python < 2.5
         teststring = 'Testing for presence of xclip.'
-        #import pdb; pdb.set_trace()
         xclipproc = subprocess.Popen('xclip -sel clip', shell=True, stdout=subprocess.PIPE, stdin=subprocess.PIPE)
         xclipproc.stdin.write(teststring)
         xclipproc.stdin.close()
@@ -130,7 +129,7 @@ class Cmd(cmd.Cmd):
     multilineCommands = []
     continuationPrompt = '> '    
     shortcuts = {'?': 'help', '!': 'shell', '@': 'load'}
-    excludeFromHistory = '''run r list l history hi ed li eof'''.split()   
+    excludeFromHistory = '''run r list l history hi ed edit li eof'''.split()   
     defaultExtension = 'txt'
     defaultFileName = 'command.txt'
     editor = os.environ.get('EDITOR')
@@ -140,7 +139,7 @@ class Cmd(cmd.Cmd):
             editor = 'notepad'
         else:
             for editor in ['gedit', 'kate', 'vim', 'emacs', 'nano', 'pico']:
-                if os.system('which %s' % (editor)):
+                if not os.system('which %s' % (editor)):
                     break
             
     settable = ['prompt', 'continuationPrompt', 'defaultFileName', 'editor', 'caseInsensitive']
@@ -175,11 +174,17 @@ class Cmd(cmd.Cmd):
         result = "\n".join('%s: %s' % (sc[0], sc[1]) for sc in self.shortcuts.items())
         self.stdout.write("Single-key shortcuts for other commands:\n%s\n" % (result))
 
-    pipeFinder = pyparsing.SkipTo(pyparsing.Literal('|') ^ pyparsing.StringEnd())
-    pipeFinder.ignore(pyparsing.sglQuotedString)
-    pipeFinder.ignore(pyparsing.dblQuotedString)
-    pipeFinder.ignore("--" + pyparsing.ZeroOrMore(pyparsing.CharsNotIn("\n"))) # sql-style comment
-    pipeFinder.ignore(pyparsing.cStyleComment)
+    notAPipe = pyparsing.SkipTo('|')
+    notAPipe.ignore(pyparsing.sglQuotedString)
+    notAPipe.ignore(pyparsing.dblQuotedString)    
+    pipeFinder = notAPipe + '|' + pyparsing.SkipTo(pyparsing.StringEnd())
+    def parsePipe(self, statement, mustBeTerminated):
+        try:
+            statement, pipe, destination = self.pipeFinder.parseString(statement)
+            redirect = subprocess.Popen(destination, shell=True, stdout=subprocess.PIPE, stdin=subprocess.PIPE)
+            return statement, redirect
+        except pyparsing.ParseException:
+            return statement, None
     
     legalFileName = re.compile(r'''^[^"'\s]+$''')
     def parseRedirector(self, statement, symbol, mustBeTerminated=False):
@@ -208,6 +213,9 @@ class Cmd(cmd.Cmd):
     
     def parseRedirectors(self, statement):
         mustBeTerminated = self.extractCommand(statement)[0] in self.multilineCommands
+        newStatement, redirect = self.parsePipe(statement, mustBeTerminated)
+        if redirect:
+            return newStatement, redirect, 'pipe'
         newStatement, redirect = self.parseRedirector(statement, '>>', mustBeTerminated)
         if redirect:            
             return newStatement, redirect, 'a'        
@@ -230,13 +238,16 @@ class Cmd(cmd.Cmd):
 
         """
         command, args = self.extractCommand(line)
-        statement = ' '.join([command, args])
+        statement = originalStatement = ' '.join([command, args])
         if (not assumeComplete) and (command in self.multilineCommands):
             statement = self.finishStatement(statement)
         statekeeper = None
         stop = 0
         statement, redirect, mode = self.parseRedirectors(statement)
-        if redirect == self._TO_PASTE_BUFFER:
+        if isinstance(redirect, subprocess.Popen):
+            statekeeper = Statekeeper(self, ('stdout',))   
+            self.stdout = redirect.stdin
+        elif redirect == self._TO_PASTE_BUFFER:
             try:
                 clipcontents = getPasteBuffer()
                 if mode in ('w', 'a'):
@@ -255,22 +266,28 @@ class Cmd(cmd.Cmd):
                 self.stdout = open(redirect, mode)            
             else:
                 statement = '%s %s' % (statement, self.fileimport(statement=statement, source=redirect))
-        stop = cmd.Cmd.onecmd(self, statement)
+        if isinstance(redirect, subprocess.Popen):
+            stop = self.onecmd(statement)
+        else:
+            stop = cmd.Cmd.onecmd(self, statement)
         try:
             if command not in self.excludeFromHistory:
-                self.history.append(statement)
+                self.history.append(originalStatement)
         finally:
             if statekeeper:
                 if redirect == self._TO_PASTE_BUFFER:
                     self.stdout.seek(0)
                     writeToPasteBuffer(self.stdout.read())
+                elif isinstance(redirect, subprocess.Popen):
+                    for result in redirect.communicate():              
+                        statekeeper.stdout.write(result or '')                        
                 self.stdout.close()
                 statekeeper.restore()
+                                 
             return stop        
         
     statementEndPattern = re.compile(r'[%s]\s*$' % terminators)        
     def statementHasEnded(self, lines):
-        #import pdb; pdb.set_trace()
         return bool(self.statementEndPattern.search(lines)) \
                or lines[-3:] == 'EOF' \
                or self.parseRedirectors(lines)[1]
@@ -476,7 +493,7 @@ class Cmd(cmd.Cmd):
             f.close()        
                 
         os.system('%s %s' % (self.editor, filename))
-        self.do_load(filename)
+        self.do__load(filename)
     do_edit = do_ed
     
     def do_save(self, fname=None):
@@ -514,7 +531,8 @@ class Cmd(cmd.Cmd):
         self.stdin.close()
         keepstate.restore()
         self.lastcmd = ''
-        return (stop == self._STOP_AND_EXIT) and self._STOP_AND_EXIT
+        return (stop == self._STOP_AND_EXIT) and self._STOP_AND_EXIT    
+    do__load = do_load  # avoid an unfortunate legacy use of do_load from sqlpython
     
     def do_run(self, arg):
         """run [arg]: re-runs an earlier command
